@@ -1,6 +1,6 @@
 #  You may distribute under the terms of the GNU General Public License
 #
-#  (C) Paul Evans, 2010-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2015 -- leonerd@leonerd.org.uk
 
 package Circle::FE::Term::Widget::Scroller;
 
@@ -40,26 +40,26 @@ sub build
 
    $widget->set_on_scrolled( sub { $self->maybe_request_more if $_[1] < 0 } );
 
-   $obj->watch_property(
-      property => "displayevents",
-      iter_from => "last",
-      on_iter => sub {
+   $tab->adopt_future(
+      $obj->watch_property_with_iter(
+         "displayevents", "last",
+         on_set => sub {
+            die "This should not happen\n";
+         },
+         on_push => sub {
+            $self->insert_event( bottom => $_ ) for @_;
+         },
+         on_shift => sub {
+            my ( $count ) = @_;
+            $count -= $self->{iter_idx};
+            $widget->shift( $count ) if $count > 0;
+         },
+      )->then( sub {
          ( $self->{iter}, undef, my $max ) = @_;
          $self->{iter_idx} = $max + 1;
 
          $self->maybe_request_more;
-      },
-      on_set => sub {
-         die "This should not happen\n";
-      },
-      on_push => sub {
-         $self->insert_event( bottom => $_ ) for @_;
-      },
-      on_shift => sub {
-         my ( $count ) = @_;
-         $count -= $self->{iter_idx};
-         $widget->shift( $count ) if $count > 0;
-      },
+      })
    );
 
    return $widget;
@@ -88,17 +88,17 @@ sub maybe_request_more
    $self->{iter_fetching} = 1;
    $widget->set_loading( 1 );
 
-   $self->{iter}->next_backward(
-      count => $more,
-      on_more => sub {
+   my $f = $self->{iter}->next_backward( $more )
+      ->on_done( sub {
          ( $self->{iter_idx}, my @more ) = @_;
 
          $self->{iter_fetching} = 0;
 
          $self->insert_event( top => $_ ) for reverse @more;
          $self->maybe_request_more;
-      },
-   );
+      });
+
+   $self->{tab}->adopt_future( $f );
 }
 
 sub insert_event
